# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from ogr.abstract import GitTag, Release
from ogr.services import pagure as ogr_pagure
from ogr.exceptions import OperationNotSupported


class PagureRelease(Release):
    project: "ogr_pagure.PagureProject"

    def __init__(
        self,
        tag_name: str,
        url: str,
        created_at: str,
        tarball_url: str,
        git_tag: GitTag,
        project: "ogr_pagure.PagureProject",
    ) -> None:
        super().__init__(tag_name, url, created_at, tarball_url, git_tag, project)

    @property
    def title(self):
        return self.git_tag.name

    @property
    def body(self):
        return ""

    def edit_release(self, name: str, message: str) -> None:
        raise OperationNotSupported("edit_release not supported on Pagure")



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from typing import Optional

from ogr.abstract import Release, GitTag
from ogr.services import gitlab as ogr_gitlab
from ogr.exceptions import OperationNotSupported


class GitlabRelease(Release):
    project: "ogr_gitlab.GitlabProject"

    def __init__(
        self,
        tag_name: str,
        url: Optional[str],
        created_at: str,
        tarball_url: str,
        git_tag: GitTag,
        project: "ogr_gitlab.GitlabProject",
        raw_release,
    ) -> None:
        super().__init__(tag_name, url, created_at, tarball_url, git_tag, project)
        self.raw_release = raw_release

    @property
    def title(self):
        return self.raw_release.name

    @property
    def body(self):
        return self.raw_release.description

    def edit_release(self, name: str, message: str) -> None:
        raise OperationNotSupported("edit_release not supported on GitLab")



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from typing import Optional, Dict, Any

from ogr.deprecation import deprecate_and_set_removal


class OgrException(Exception):
    """Something went wrong during our execution."""

    pass


class PagureAPIException(OgrException):
    """Exception related to Pagure API."""

    def __init__(
        self,
        *args: Any,
        pagure_error: Optional[str] = None,
        pagure_response: Optional[Dict[str, Any]] = None
    ) -> None:
        super().__init__(*args)
        self._pagure_error = pagure_error
        self.pagure_response = pagure_response

    @property
    def pagure_error(self):
        return self._pagure_error or self.__cause__


class GithubAPIException(OgrException):
    """Exception related to Github API."""

    @property  # type: ignore
    @deprecate_and_set_removal(
        since="0.30.0",
        remove_in="0.35.0 (or 1.0.0 if it comes sooner)",
        message="Use __cause__",
    )
    def github_error(self):
        return self.__cause__


class GitlabAPIException(OgrException):
    """Exception related to Gitlab API."""

    @property  # type: ignore
    @deprecate_and_set_removal(
        since="0.30.0",
        remove_in="0.35.0 (or 1.0.0 if it comes sooner)",
        message="Use __cause__",
    )
    def gitlab_error(self):
        return self.__cause__


class OperationNotSupported(OgrException):
    """Raise when the operation is not supported by the backend."""


class OgrNetworkError(OgrException):
    """Exception raised when an unexpected network error occurs."""



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from github.GitRelease import GitRelease as PyGithubRelease

from ogr.abstract import Release, GitTag
from ogr.services import github as ogr_github


class GithubRelease(Release):
    project: "ogr_github.GithubProject"

    def __init__(
        self,
        tag_name: str,
        url: str,
        created_at: str,
        tarball_url: str,
        git_tag: GitTag,
        project: "ogr_github.GithubProject",
        raw_release: PyGithubRelease,
    ) -> None:
        super().__init__(tag_name, url, created_at, tarball_url, git_tag, project)
        self.raw_release = raw_release

    @property
    def title(self):
        return self.raw_release.title

    @property
    def body(self):
        return self.raw_release.body

    def edit_release(self, name: str, message: str) -> None:
        """
        Edit name and message of a release.

        Args:
            name: New name of the release.
            message: New message for the release.
        """
        self.raw_release = self.raw_release.update_release(name=name, message=message)



from datetime import datetime

from requre.online_replacing import record_requests_for_all_methods

from tests.integration.gitlab.base import GitlabTests


@record_requests_for_all_methods()
class Comments(GitlabTests):
    def test_pr_react_to_comment_and_delete(self):
        pr = self.service.get_project(repo="playground", namespace="nikromen").get_pr(2)
        pr_comment = pr.comment(datetime.now().strftime("%m/%d/%Y"))

        reaction = pr_comment.add_reaction("+1")
        assert len(pr_comment.get_reactions()) == 1

        reaction.delete()
        assert len(pr_comment.get_reactions()) == 0

    def test_issue_react_to_comment_and_delete(self):
        issue = self.service.get_project(
            repo="playground", namespace="nikromen"
        ).get_issue(2)
        issue_comment = issue.comment(datetime.now().strftime("%m/%d/%Y"))

        reaction = issue_comment.add_reaction("tractor")
        assert len(issue_comment.get_reactions()) == 1

        reaction.delete()
        assert len(issue_comment.get_reactions()) == 0

    def test_get_reactions(self):
        pr = self.service.get_project(repo="playground", namespace="nikromen").get_pr(2)
        pr_comment = pr.comment(datetime.now().strftime("%m/%d/%Y"))

        pr_comment.add_reaction("+1")
        pr_comment.add_reaction("-1")
        pr_comment.add_reaction("tractor")
        assert len(pr_comment.get_reactions()) == 3

        issue = self.service.get_project(
            repo="playground", namespace="nikromen"
        ).get_issue(2)
        issue_comment = issue.comment(datetime.now().strftime("%m/%d/%Y"))

        issue_comment.add_reaction("+1")
        issue_comment.add_reaction("-1")
        issue_comment.add_reaction("tractor")
        assert len(pr_comment.get_reactions()) == 3
